#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h,
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e1[0457]|c2>:<[nofcpp|mpich|mvapich2]> [debug|prof] [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    (( seen_bqual = 0 ))
    for qual in "${qlist[@]}"; do
        case $qual in
            c[1-9]*)
                cqual=$qual
                ccdef=clang
                cxxdef=clang++
                fcdef=gfortran
               ;;
            e[1-9]*)
                cqual=$qual
                ccdef=gcc
                cxxdef=g++
                fcdef=gfortran
                ;;
            mpich|mvapich2)
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich, mvapich2, and nofcpp are mutually exclusive"
                bqual=$qual
                cmakeoptions=(-DHDF5_BUILD_CPP_LIB:BOOL=OFF \
                  -DHDF5_BUILD_FORTRAN:BOOL=ON \
                  -DHDF5_ENABLE_PARLLEL:BOOL=ON \
                  -DHDF5_ENABLE_Z_LIB_SUPPORT:BOOL=ON)
                confoptions=(--disable-cxx --enable-fortran --disable-dependency-tracking --enable-parallel)
                cc=mpicc
                fc=mpif90
                cxx=""
                [[ "${bqual}" == mvapich2 ]] && (( HDF5_NO_CHECKS = 1 ))
                ;;
            nofcpp)
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich, mvapich2, and nofcpp are mutually exclusive"
                bqual=$qual
                cmakeoptions=(-DHDF5_BUILD_CPP_LIB:BOOL=OFF -DHDF5_BUILD_FORTRAN:BOOL=OFF)
                confoptions=(--disable-cxx --disable-fortran --disable-dependency-tracking)
                cxx=""
                fc=""
                ;;
            *)
                ssi_die "Unrecognized qualifier $qual"
        esac
    done

    if [[ -z "${bqual}" ]]; then
      cmakeoptions=(-DHDF5_BUILD_CPP_LIB:BOOL=ON -DHDF5_BUILD_FORTRAN:BOOL=ON)
      confoptions=(--enable-cxx --enable-fortran --disable-dependency-tracking)
      define_basics_args="NO_TYPE"
    fi

    if [[ -z "$cqual" ]]; then
        ssi_die "Base qualifier is mandatory"
    fi

    if [[ -z "${extraqual}" ]] && [[ -n "${bqual}" ]]; then
      ssi_die "debug|prof is mandatory."
    elif [[ -z "${bqual}" ]] && [[ -n "${extraqual}" ]]; then
      ssi_die "$extraqual not compatible with current configuration."
    fi
    basequal=${cqual}${bqual:+:${bqual}}
 }

set_compiler_flags()
{
  case ${cqual} in
    e6)
      cmakeoptions+=(-DCMAKE_CXX_STANDARD=14)
      cxxflags="-std=c++1y"
      ;;
    e[79]|e1[045])
      cmakeoptions+=(-DCMAKE_CXX_STANDARD=14)
      cxxflags="-std=c++14"
      ;;
    e17|c[24])
      cmakeoptions+=(-DCMAKE_CXX_STANDARD=17)
      cxxflags="-std=c++17"
      ;;
    *)
      ssi_die "Unrecognized compiler qualifier $cqual"
  esac
  case x$extraqual in
    xdebug)
      cmakeoptions+=(-DCMAKE_BUILD_TYPE=Debug \
        -DHDF5_ENABLE_USING_MEMCHECKER:BOOL=ON \
        -DHDF5_MEMORY_ALLOC_SANITY_CHECK:BOOL=ON \
        -DHDF5_ENABLE_DEBUG_APIS:BOOL=ON \
        -DHDF5_ENABLE_TRACE:BOOL=ON)
      confoptions+=(--enable-build-mode=debug \
        --enable-using-memchecker \
        --enable-memory-alloc-sanity-check \
        --enable-internal-debug=all \
        --enable-trace=yes)
      ;;
    xprof|x)
      cmakeoptions+=(-DCMAKE_BUILD_TYPE=RelWithDebInfo)
      confoptions+=(--enable-build-mode=production \
        --enable-symbols=yes)
      ;;
    *)
      ssi_die "Unrecognized extra qualifier $extraqual"
  esac
  # Temporary.
  cmakeoptions+=(-DHDF5_BUILD_HL_LIB:BOOL=OFF)
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------
(( want_cmake = ${USE_CMAKE:-0} ))

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
if [[ "$extraqual" == "tar" ]]; then
  unset extraqual
  maketar=tar
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=hdf5
origpkgver=v1_10_2
pkgver=${origpkgver//-/_}a
ssibuildshims_version=v1_04_05
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims

set -- # Clear positional parameters.

source ssi_functions # For ssi_die().

set_qualifiers
source define_basics $define_basics_args
set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

# Build-time-only dependencies
if (( ${want_cmake:-0} )); then
  cmakeprod=$(latest_version -t ${product_dir} cmake -z "${product_dir}" -z "${PRODUCTS}")
  [[ -n "${cmakeprod}" ]] || ssi_die "Unable to find suitable version of cmake"
  eval setup -B ${cmakeprod} || ssi_die "Unable to set up ${cmakeprod}"
fi

ups active

set -x

cd "${sourcedir}" && rm -rf hdf5-${pkgdotver} || ssi_die "Unable to cd into ${sourcedir}"
tar xf ${tardir}/${pkgtarname}* || ssi_die "Unable to untar source"

# Apply patch.
#cd "${pkgtarname}" || ssi_die "Unable to cd into ${sourcedir}/${pkgtarname} for application of patch."
#patch="hdf51.10-CVE2016.patch"
#patch ${DEBUG_PATCHES:+--verbose} -p0 < "${patchdir}/${patch}" || ssi_die "application of patch ${patch} failed."
[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd "${blddir}" || ssi_die "Unable to cd into ${blddir}"

# SWMR checks *still* don't work on Darwin.
[[ "$OS1" == "Darwin" ]] && (( HDF5_NO_CHECKS = 1 ))

if (( ${want_cmake:-0} )); then
  eval CC=${cc-${ccdef}} CXX=${cxx-${cxxdef}} FC=${fc-${fcdef}} \
    cmake -DCMAKE_INSTALL_PREFIX="${pkgdir}" "${cmakeoptions[@]}" \
    ${sourcedir}/hdf5-${pkgdotver} || \
    ssi_die "cmake execution failed"
else
  eval CC=${cc-${ccdef}} CXX=${cxx-${cxxdef}} FC=${fc-${fcdef}} \
    ${gflag:+CFLAGS=${gflag}} \
    ${cxxflags:+CXXFLAGS=\""${cxxflags}"\"} \
    ${sourcedir}/hdf5-${pkgdotver}/configure --prefix="${pkgdir}" "${confoptions[@]}" || \
    ssi_die "configure failed"
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

make -j ${ncore} || ssi_die "make failed"

if ! (( ${HDF5_NO_CHECKS:-0} )); then
  if (( ${want_cmake:-0} )); then
    ctest -j ${ncore} --output-on-failure || ssi_die "Tests failed"
  else
    make -j ${ncore} check || ssi_die "make check failed"
  fi
fi

make -j ${ncore} install || ssi_die "make install failed"

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# In 1.10.1, some unwanted libs were installed:
rm -f "${pkgdir}/lib/libdynlib"*

# Fix rpaths in Fortran libraries.
flibs=(libhdf5_{,hl_}fortran.dylib)
if [[ "${OS1}" == "Darwin" ]]; then
  pushd "${pkgdir}/lib"
  for flib in "${flibs[@]}"; do
    if [[ -f "${pkgdir}/lib/${flib}" ]]; then
      otool -L "${flib}"
      fix_macos_lib_symbols "${flib}"
    fi
  done
  popd
fi

# Update wrapper with correct prefix.
for f in ${pkgdir}/bin/h5{,p}cc ${pkgdir}/bin/h5c++ ${pkgdir}/bin/h5fc ; do
  [[ -w "${f}" ]] || continue
  perl -wapi\~ -e 's&^prefix=.*$&prefix="\${HDF5_FQ_DIR}"&' "${f}" || \
    ssi_die "Unable to update wrapper scripts." && rm -f "${f}~"
done
# remove libtool files since they contain hardwired paths
rm -vf ${pkgdir}/lib/*.la

set +x

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "hdf5 is installed at ${HDF5_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
